#Texture Aligner (for horizontal faces)

#Author:	Eneroth3, eneroth3@gmail.com

#Combability
#	Coded and tested on windows7, Sketchup 8

#Usage
#	Menu: Plugins
#	Toolbar: Texture alignment
#		Align Textures To Individual Face's Side:	Align texture of each face to one of its sides, usually the one first drawn.
#		Align Textures To Selected Edge:			Align texture of faces to selected line. Only one line can be selected.

#Copyright Julia Christina Eneroth (eneroh3)
#	Free to use both professinal and unprofessional.
#	Free to spread as long as credit goes to me.
#	Please link to my website when publishing online (http://eneroth.port0.org/plugins/texturealigner/).

#Known bugs
#	<None>

#Potential bugs
#	<none>

#Changelog
#	2013-02-24
#		*First release

#require 'sketchup.rb'#REQUIRED??

module Ene_alignTexture

plugins_menu = UI.menu("Plugins")

plugins_menu.add_item("Align Textures To Individual Face's Side") { self.align_material }
plugins_menu.add_item("Align Textures To Selected Edge") { self.align_material(true) }

tb = UI::Toolbar.new "Texture Alignment"

cmd = UI::Command.new("Align Texture") { self.align_material }
cmd.large_icon = cmd.small_icon = "ene_alignTexture/undefinedEdge.png"
cmd.status_bar_text = cmd.tooltip = "Align Textures To Individual Face's Side"
cmd.set_validation_proc {							#I might change when the tools are enabled or not
	if Sketchup.active_model.selection.length == 0
		MF_GRAYED
	else
		MF_ENABLED
	end
}
tb.add_item cmd

cmd = UI::Command.new("Align Texture To Edge") { self.align_material(true) }
cmd.large_icon = cmd.small_icon = "ene_alignTexture/definedEdge.png"
cmd.status_bar_text = cmd.tooltip = "Align Textures To Selected Edge"
cmd.set_validation_proc {
	if Sketchup.active_model.selection.length == 0
		MF_GRAYED
	else
		MF_ENABLED
	end
}
tb.add_item cmd

def self.align_material(definedEdge = false)
	#When definedEdge is false all face's textures are aligned to their own first bounding edge.
	#When true all face's textures are aligned to the one selected edge. Only one edge can then be selected
	
	ss = Sketchup.active_model.selection
	
	if definedEdge then									###Align to predefined (selected) edge

		ssEdges = []									#Put selected edges in there own array
		ss.each do |i|
			next unless i.class == Sketchup::Edge
			ssEdges << i
		end#each
		unless ssEdges[0]								#At least one edge must be selected
			UI.messagebox("An edge to align to must be selected.")
			return
		end#if
		if ssEdges[1] then								#More than one edge can't be selected
			UI.messagebox("Can only align to one edge. Select one and retry.")
			ss.toggle ssEdges							#Deselect all edges so user can select one (just one) and retry
			return
		end#if
		globalVector = ssEdges[0].line[1]				#Edge to align all faces too
		
	end#if

	ss.each do |i|
	
		next unless i.class == Sketchup::Face			#Skip everything but faces
		next unless i.material							#Skip if no material or no texture
		next unless i.material.texture
		vector = i.edges[0].line[1] unless globalVector	#Align to first bounding edge if no edge is defined
		vector = globalVector if globalVector			#Align to predefined edge
		next unless vector[2] < 0.000001				#Skip if face is't horizontal (some random tolerance)
		achorPoint = i.edges[0].line[0]					#Point to rotate around
			
		textureWidth = i.material.texture.width
		vector.length = textureWidth					#Change vector's length to materials width
		
		points = [achorPoint, [0,0,0], [achorPoint[0]+vector[0],achorPoint[1]+vector[1],achorPoint[2]+vector[2]], [1,0,0]]
		i.position_material(i.material, points, true)
		
	end#each

end#def

end#module